c
c**************************************************
c**************************************************
c
c	Zemlyanaya E.V.   Version 15.03.2005
c
c CONTINUATION CODE FOR NONLINEAR SYSTEM OF TWO ODEs
c
c**************************************************
c**************************************************
c*
	program CONTIN_NLIN

      implicit real * 8(a-h,o-z)
	parameter (nd= 50001)
      dimension u2(nd),u1(nd),x(nd),r1(nd),r2(nd),r3(nd)
      dimension r4(nd),bm(2,nd),u1p(nd),u2p(nd),p(2,2,nd),pp(2,2,nd)
      dimension f(2,2,nd),uk(2,nd),d1(2),d2(2),f1(2),f2(2),g1(2)
      dimension g2(2),v(2,nd),am(2,2,nd)

	common /param/ alpha(100)
ccc
ccc
	open(3,file='param', status='old')

	open(13,file='f_out',status='unknown')
	open( 1,file='e_out',status='unknown')

c	read 1st line of input file (vector of parameters)
c	npar  -- number of parameters
c	alpha -- vector of parameters
c	nc    -- number of the continuaetion parameter

c	read 2st line of input file (parameters of continuation)
c	steph -- initial stepsize of continuation
c	nstep -- if nstep=0 then steph=steph0=constant else steph is calculated
c	eps_s -- condition of turning
c	nsmax -- maximal number of continuation steps

c	read 3nd line of input file (parameters of Newtonian scheme)
c	tau0 -- initial value of tau
c	ntau -- number of algorithm for tau (only 1,2,3)	
c	eps	 -- condintion of convergence
c	eps_max -- if residual > eps_max then stop
c	nmax --  maximal number of newtonian iterations
c
c	read 4th line (parameters of initial guess)
c	n    -- number of points
c	rmin -- left side of interval
c	rmax -- right side of interval
c	ini  -- if ini=0 then initial guess is calculated by subroutine f_init 	
c			if ini=1 then initial guess is taken from the file in_guess
c
c	read 5th line (boundar conditions on the left)
c	boundary conditions on the left: d1(1),f1(1),d1(2),f1(2)

c	read 6th line (boundar conditions on the left)
c	boundary conditions on the right: d2(1),f2(1),d2(2),f2(2)
     
       read (3,*) npar, (alpha(I),i=1,npar), nc
	 read (3,*) steph, nstep,eps_s,nsmax
       read (3,*) tau0,ntau,eps,eps_max,nmax
	 read (3,*) n,a,b,ini 
	 read (3,*) d1(1),f1(1),d1(2),f1(2)
	 read (3,*) d2(1),f2(1),d2(2),f2(2)
ccc

       nm1   = n-1
       h     = (b-a)/nm1
	print *,h

	ns = 1

       do i  = 1,n
       x(i)  = a+h*(i-1)
  	 end do

	if(ini.ne.0) then
	open(33,file='init_guess',status='old')
	do i=1,n
      READ (33,*)XXX,U1(I),U2(I)
	u1p(i) = u1(i)
	u2p(i) = u2(i)
      end do
c
	else 
      call f_init(n,U1,U2,x)
	do i=1,n
	u1p(i) = u1(i)
	u2p(i) = u2(i)
      end do
	end if
c
	call bif_m(n,h,u1,u2,r1,e)
       ep    = e
       print *,'energy=', e
c
c----------------------------------- begin continuation
c
 600   continue

       nit   = 0
       print * ,'******************************** ns=',ns

       gamma = alpha(1)
       hs    = alpha(2)
	 vel = alpha(3)
	print *,(alpha(i),i=1,npar)

c
       tau   = tau0
       tp    = tau0
       delmax= 0.d0
c
c----------------------------------- begin newtonian iterations
c
 400   continue

       call d1t5 (n,x,u1,r1)
       call d1t5 (n,x,r1,r2)
       call d1t5 (n,x,u2,r3)
       call d1t5 (n,x,r3,r4)
c

	call ggg(n,u1,u2,x,uk) 
	call fff(n,u1,u2,x,p) 

       do i  = 2,nm1
	zzz = p(2,1,i)
	p(2,1,i) = p(2,2,i)
	p(2,2,i) = zzz

c	print *,i,p(1,1,i),p(1,2,i),p(2,1,i),p(2,2,i)
c	pause
	uk(1,i) = uk(1,i) + r2(i) +p(1,1,i)*r1(i) + p(1,2,i)*r3(i)
      uk(2,i) = uk(2,i) + r4(i)+ p(2,1,i)*r3(i) + p(2,2,i)*r1(i)

       absdel1  = max(abs(uk(1,i)),delmax)
       absdel2  = max(abs(uk(2,i)),delmax)

	    if (absdel1.gt.delmax) then
       delmax= absdel1
       xmax  = x(i)
       imax  = 1
		end if
		if (absdel2.gt.delmax) then
       delmax= absdel2
       xmax  = x(i)
       imax  = 2
		end if
       end do
       print 2001,nit,delmax,xmax,imax

       if(delmax.lt.eps)go to 100
       if(delmax.gt.eps_max) go to 200
c
       if(nit.gt.nmax) go to 100
c
       g1(1) = -(d1(1)*r1(1)+f1(1)*u1(1))
       g2(1) = -(d2(1)*r1(n)+f2(1)*u1(n))
       g1(2) = -(d1(2)*r3(1)+f1(2)*u1(1))
       g2(2) = -(d2(2)*r3(n)+f2(2)*u2(n))
c

	call dgdu(n,u1,u2,x,f)
	
      do i  = 1,n
	zzz = f(2,1,i)
	f(2,1,i) = f(2,2,i)
	f(2,2,i) = zzz	
      end do
c
       do i  = 2,nm1

	f(1,1,i) = f(1,1,i) + df11du1(u1(i),u2(i),x(i))*r1(i)+
     ^				df12du1(u1(i),u2(i),x(i))*r3(i)

	f(1,2,i) = f(1,2,i) + df11du2(u1(i),u2(i),x(i))*r1(i)+
     ^				df12du2(u1(i),u2(i),x(i))*r3(i)

	f(2,1,i) = f(2,1,i) + df21du2(u1(i),u2(i),x(i))*r1(i)+
     ^				df22du2(u1(i),u2(i),x(i))*r3(i)

	f(2,2,i) = f(2,2,i) + df21du1(u1(i),u2(i),x(i))*r1(i)+
     ^				df22du1(u1(i),u2(i),x(i))*r3(i)

       uk(1,i)  = -uk(1,i)
       uk(2,i)  = -uk(2,i)
       end do
c
       do j  = 1,2
       do i  = 1,n
       bm(j,i)  = 0.d0
       do k  = 1,2
       am(k,j,i)= 0.d0
       end do
       end do
       end do
ccc
	call progs2h4p(n,h,f,p,uk,d1,d2,f1,f2,g1,g2,v,am,bm)
ccc 
      call tauka(NTAU,TP,Dp,Delmax,TAU0,TAU)
ccc
       do i  = 1,n
       u1(i) = u1(i)+v(1,i)*tau
       u2(i) = u2(i)+v(2,i)*tau
       end do
ccc
       nit   = nit+1
       dp    = delmax
       delmax= 0.
       tp    = tau
       go to 400
 100   print * ,' exit: delmax < eps'
 
c      go to 800
       rewind 13
       do i  = 1,n
       write (13,1111) x(i),u1(i),u2(i)
       end do
c
       go to 500
 200   print * ,' exit: nit > nmax'
       go to 500

 500   continue
c
c----------------------------------- end of newtonian iterations
c
       call bif_m(n,h,u1,u2,r1,e)
       print *,'energy=',e
       delu  = e-ep

      if(ns.ge.2) then
       dhds  = steph/delu
       print *,'============== ns,steph,dh/ds=',ns,steph,delu,dhds
       if(dabs(dhds).le.eps_s)then
       steph = -steph
       ns    = 0
c	    tau0 = .10
c		tp  = .1
c	    ntau  = 1

       print *,'!!!!!! turning !!!!!!','dh/ds=',dhds
	 pause
      end if
	end if

      write (1,1001) alpha(nc),e,dhds

ccc
      do i   = 1,n
      r1(i)  = u1(i)
      r2(i)  = u2(i)
c
      u1(i)  = 2.d0*u1(i)-u1p(i)
      u2(i)  = 2.d0*u2(i)-u2p(i)
c
      u1p(i) = r1(i)
      u2p(i) = r2(i)
      end do
c

      if (nstep.ne.0.and.ns.gt.2)steph = steph/delu*dep

	alpha(nc)     = alpha(nc)+steph

	print *,'!!!',hs,delu,dep,steph
c	pause
	
      ns  = ns+1
	dep = delu
      ep  = e
	
      if(ns.le.nsmax) go to 600

 800  continue


 1111  format(1x,f15.7,3(1x,e25.15))
 1001  format(3(1x,e20.10))
 2001  format(' nit,delmax,xmax,l=',i5,e17.10,1x,f10.3,i5)
 
      END
c*
c**************************************************
c*
*
c***********************************************************************
c
      subroutine progs2h4p(n,h,f,p,uk,d1,d2,f1,f2,g1,g2,y,am,bm)
c
c------------------------------------------------------------------
c     Zemlyanaya E.V., Puzynina T.P.
c	Last correction: 04.03.02
c
c     Solution of boundary problem for the diff. system (two eqs.)
c     the 4th order accuracy is provided by Numerov scheme
c------------------------------------------------------------------
c
      implicit real*8 (a-h,o-z)
      dimension f(2,2,n),p(2,2,n),uk(2,n),d1(2),d2(2),f1(2),f2(2),g1(2),
     ^          g2(2),y(2,n),am(2,2,n),bm(2,n),e(10,11),index(11)


	l = 2

      n1   = n-1
      n2   = n-2
      n3   = n-3
      n4   = n-4
      n5   = n-5
      hh   = h*h
      h7d6 = hh*7.0d0/6.0d0
      h5d12= hh*5.0d0/12.0d0
      hd3  = hh/3.0d0
      hd12 = hh/12.0d0
      h5d6 = hh*5.0d0/6.0d0

      h3p12= h*hh/12.0d0
      h12  = 12.0d0*h
	h524 = 5.0d0*h/24.0d0
	h712 = 7.0d0*h/12.0d0
    	h6=h/6.0d0
	h8=h/8.0d0
    	h24  = h/24.0d0

      do i  = 1,n
      do j  = 1,l
      bm(j,i)  = 0.
      do k  = 1,l
      am(k,j,i)= 0.
      end do
      end do
      end do


c--------------------------------------- the left system
c
c--------------------------------- i=1
c      print *,'c--------------------------------- i=1'
c
      a1   = -25.0d0*d1(1)+h12*f1(1)
      b1   = 48.0d0*d1(1)
      c1   = -36.0d0*d1(1)
      dd1  = 16.0d0*d1(1)
      e1   = -3.0d0*d1(1)
      uk1h = h12*g1(1)
c
      a1v  = -25.0d0*d1(2)+h12*f1(2)
      b1v  = 48.0d0*d1(2)
      c1v  = -36.0d0*d1(2)
      dd1v = 16.0d0*d1(2)
      e1v  = -3.0d0*d1(2)
      uk1hv= h12*g1(2)


c
c--------------------------------- i=2
c      print *,'c--------------------------------- i=2'
c
      a2   = 1.
      b2   = -2.0d0+h7d6*f(1,1,2)
      c2   = 1.0d0-h5d12*f(1,1,3)
      dd2  = hd3*f(1,1,4)
      e2   = -hd12*f(1,1,5)
      b2c  = h7d6*f(1,2,2)
      c2c  = -h5d12*f(1,2,3)
      dd2c = hd3*f(1,2,4)
      e2c  = -hd12*f(1,2,5)
      uk2h = uk(1,2)*h7d6-h5d12*uk(1,3)+hd3*uk(1,4)-hd12*uk(1,5)


      a2v  = 1.0d0
      b2v  = -2.0d0+h7d6*f(2,1,2)
      c2v  = 1.0d0-h5d12*f(2,1,3)
      dd2v = hd3*f(2,1,4)
      e2v  = -hd12*f(2,1,5)
      b2cv = h7d6*f(2,2,2)
      c2cv = -h5d12*f(2,2,3)
      dd2cv= hd3*f(2,2,4)
      e2cv = -hd12*f(2,2,5)
      uk2hv= uk(2,2)*h7d6-h5d12*uk(2,3)+hd3*uk(2,4)-hd12*uk(2,5)

corr for general case
	a2c  = 0.d0
	a2cv = 0.d0


c
c for system with first derivatives.  (begin corrections 1, formulas 24,18)
c
	r    = -1.50d0*p(1,1,2)/h
	s    = 3.0d0*f(1,1,2)+2.0d0*p(1,1,3)/h
	u    = 1.50d0*p(1,1,2)/h-4.0d0*f(1,1,3)-0.50d0*p(1,1,4)/h
	v    = -2.0d0*p(1,1,3)/h+f(1,1,4)
	w    = 0.50d0*p(1,1,4)/h

	rc   = -1.50d0*p(1,2,2)/h
	sc   = 3.0d0*f(1,2,2)+2.0d0*p(1,2,3)/h
	uc   = 1.50d0*p(1,2,2)/h-4.0d0*f(1,2,3)-0.50d0*p(1,2,4)/h
	vc   = -2.0d0*p(1,2,3)/h+f(1,2,4)
	wc   = 0.50d0*p(1,2,4)/h
	t    = -3.0d0*uk(1,2)+4.0d0*uk(1,3)-uk(1,4)

	rv   = -1.50d0*p(2,1,2)/h
	sv   = 3.0d0*f(2,1,2)+2.0d0*p(2,1,3)/h
	uv   = 1.50d0*p(2,1,2)/h-4.0d0*f(2,1,3)-0.5d0*p(2,1,4)/h
	vv   = -2.0d0*p(2,1,3)/h+f(2,1,4)
	wv   = 0.50d0*p(2,1,4)/h

	rcv  = -1.50d0*p(2,2,2)/h
	scv  = 3.0d0*f(2,2,2)+2.*p(2,2,3)/h
	ucv  = 1.50d0*p(2,2,2)/h-4.d0*f(2,2,3)-0.50d0*p(2,2,4)/h
	vcv  = -2.0d0*p(2,2,3)/h+f(2,2,4)
	wcv  = 0.50d0*p(2,2,4)/h
	tv   = -3.0d0*uk(2,2)+4.0d0*uk(2,3)-uk(2,4)


	a2   = a2-h712*p(1,1,2)-h3p12*(p(1,1,2)*r+p(1,2,2)*rcv)
	b2   = b2+h524*p(1,1,3)-h3p12*(p(1,1,2)*s+p(1,2,2)*scv)
	c2   = c2-h24*(-13.d0*p(1,1,2)+p(1,1,4)+p(1,1,5))-
     -         h3p12*(p(1,1,2)*u+p(1,2,2)*ucv)
	dd2  = dd2-h524*p(1,1,3)-h3p12*(p(1,1,2)*v+p(1,2,2)*vcv)
	e2   = e2-h24*(-4.d0*p(1,1,4)+3.d0*p(1,1,5))-
     -         h3p12*(p(1,1,2)*w+p(1,2,2)*wcv)
	a2c  = a2c-h712*p(1,2,2)-h3p12*(p(1,1,2)*rc+p(1,2,2)*rv)
	b2c  = b2c+h524*p(1,2,3)-h3p12*(p(1,1,2)*sc+p(1,2,2)*sv)
	c2c  = c2c-h24*(-13.d0*p(1,2,2)+p(1,2,4)+p(1,2,5))-
     -         h3p12*(p(1,1,2)*uc+p(1,2,2)*uv)
	dd2c = dd2c-h524*p(1,2,3)-h3p12*(p(1,1,2)*vc+p(1,2,2)*vv)
	e2c  = e2c-h24*(-4.d0*p(1,2,4)+3.d0*p(1,2,5))-
     -         h3p12*(p(1,1,2)*wc+p(1,2,2)*wv)

      uk2h = uk2h-h3p12*(p(1,1,2)*t+p(1,2,2)*tv)


	a2v  = a2v-h712*p(2,1,2)-h3p12*(p(2,1,2)*rv+p(2,2,2)*rc)
	b2v  = b2v+h524*p(2,1,3)-h3p12*(p(2,1,2)*sv+p(2,2,2)*sc)
	c2v  = c2v-h24*(-13.d0*p(2,1,2)+p(2,1,4)+p(2,1,5))-
     -         h3p12*(p(2,1,2)*uv+p(2,2,2)*uc)
	dd2v = dd2v-h524*p(2,1,3)-h3p12*(p(2,1,2)*vv+p(2,2,2)*vc)
	e2v  = e2v-h24*(-4.d0*p(2,1,4)+3.*p(2,1,5))-
     -         h3p12*(p(2,1,2)*wv+p(2,2,2)*wc)
	a2cv = a2cv-h712*p(2,2,2)-h3p12*(p(2,1,2)*rcv+p(2,2,2)*r)
	b2cv = b2cv+h524*p(2,2,3)-h3p12*(p(2,1,2)*scv+p(2,2,2)*s)
	c2cv = c2cv-h24*(-13.d0*p(2,2,2)+p(2,2,4)+p(2,2,5))-
     -         h3p12*(p(2,1,2)*ucv+p(2,2,2)*u)
	dd2cv= dd2cv-h524*p(2,2,3)-h3p12*(p(2,1,2)*vcv+p(2,2,2)*v)
	e2cv = e2cv-h24*(-4.d0*p(2,2,4)+3.*p(2,2,5))-
     -         h3p12*(p(2,1,2)*wcv+p(2,2,2)*w)

      uk2hv= uk2hv-h3p12*(p(2,1,2)*tv+p(2,2,2)*t)

c
c for system with first derivatives.  (end corrections 1)
c


c--------------------------------- i=3
c      print *,'c--------------------------------- i=3'
c
corr for general case
	i=  3
      im1=i-1
      ip1=i+1

c formulas (22)

      a3   = 1.0d0+hd12*f(1,1,im1)
      b3   = h5d6*f(1,1,i)-2.
      c3   = 1.0d0+hd12*f(1,1,ip1)
      a3c  = hd12*f(1,2,im1)
      b3c  = h5d6*f(1,2,i)
      c3c  = hd12*f(1,2,ip1)
      f3   = uk(1,i)*hh+hd12*(uk(1,ip1)-2.*uk(1,i)+uk(1,im1))
c
      a3v  = 1.0d0+hd12*f(2,1,im1)
      b3v  = h5d6*f(2,1,i)-2.0d0
      c3v  = 1.0d0+hd12*f(2,1,ip1)
      a3cv = hd12*f(2,2,im1)
      b3cv = h5d6*f(2,2,i)
      c3cv = hd12*f(2,2,ip1)
      f3v  = uk(2,i)*hh+hd12*(uk(2,ip1)-2.*uk(2,i)+uk(2,im1))

c
c for system with first derivatives.  (begin corrections 2, formulas 22,16)
c
c
      aicl  = f(1,1,im1)-0.50d0*(p(1,1,ip1)+3.0d0*p(1,1,im1))/h
      bicl  = 2.0d0*(p(1,1,ip1)+p(1,1,im1))/h
      cicl  = -f(1,1,ip1)-0.50d0*(3.*p(1,1,ip1)+p(1,1,im1))/h
      aiclc = f(1,2,im1)-0.50d0*(p(1,2,ip1)+3.*p(1,2,im1))/h
      biclc = 2.0d0*(p(1,2,ip1)+p(1,2,im1))/h
      ciclc = -f(1,2,ip1)-0.50d0*(3.*p(1,2,ip1)+p(1,2,im1))/h
      ficl  = uk(1,ip1)-uk(1,im1)
c
      aiclv = f(2,1,im1)-0.50d0*(p(2,1,ip1)+3.*p(2,1,im1))/h
      biclv = 2.0d0*(p(2,1,ip1)+p(2,1,im1))/h
      ciclv = -f(2,1,ip1)-0.50d0*(3.*p(2,1,ip1)+p(2,1,im1))/h
      aiclcv= f(2,2,im1)-0.50d0*(p(2,2,ip1)+3.*p(2,2,im1))/h
      biclcv= 2.0d0*(p(2,2,ip1)+p(2,2,im1))/h
      ciclcv= -f(2,2,ip1)-0.50d0*(3.*p(2,2,ip1)+p(2,2,im1))/h
      ficlv=  uk(2,ip1)-uk(2,im1)

      a3   = a3-h24*(-p(1,1,ip1)+10.0d0*p(1,1,i)+3.0d0*p(1,1,im1))-
     -       h3p12*(p(1,1,i)*aicl+p(1,2,i)*aiclcv)
      b3   = b3-h6*(p(1,1,ip1)-p(1,1,im1))-
     -       h3p12*(p(1,1,i)*bicl+p(1,2,i)*biclcv)
      c3   = c3-h24*(-3.0d0*p(1,1,ip1)-10.0d0*p(1,1,i)+p(1,1,im1))-
     -       h3p12*(p(1,1,i)*cicl+p(1,2,i)*ciclcv)
      a3c  = a3c-h24*(-p(1,2,ip1)+10.0d0*p(1,2,i)+3.*p(1,2,im1))-
     -       h3p12*(p(1,1,i)*aiclc+p(1,2,i)*aiclv)
      b3c  = b3c-h6*(p(1,2,ip1)-p(1,2,im1))-
     -       h3p12*(p(1,1,i)*biclc+p(1,2,i)*biclv)
      c3c  = c3c-h24*(-3.0d0*p(1,2,ip1)-10.*p(1,2,i)+p(1,2,im1))-
     -       h3p12*(p(1,1,i)*ciclc+p(1,2,i)*ciclv)

      f3   = f3-h3p12*(p(1,1,i)*ficl+p(1,2,i)*ficlv)

c
      a3v  = a3v-h24*(-p(2,1,ip1)+10.0d0*p(2,1,i)+3.*p(2,1,im1))-
     -       h3p12*(p(2,1,i)*aiclv+p(2,2,i)*aiclc)
      b3v  = b3v-h6*(p(2,1,ip1)-p(2,1,im1))-
     -       h3p12*(p(2,1,i)*biclv+p(2,2,i)*biclc)
      c3v  = c3v-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-
c     -       h3p12*(p(2,1,i)*ciclc+p(2,2,i)*ciclc)
     -       h3p12*(p(2,1,i)*ciclv+p(2,2,i)*ciclc)
      a3cv = a3cv-h24*(-p(2,2,ip1)+10.0d0*p(2,2,i)+3.*p(2,2,im1))-
     -       h3p12*(p(2,1,i)*aiclcv+p(2,2,i)*aicl)
      b3cv = b3cv-h6*(p(2,2,ip1)-p(2,2,im1))-
     -       h3p12*(p(2,1,i)*biclcv+p(2,2,i)*bicl)

corr 17.07.97
c      c3cv = c3cv-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-
      c3cv = c3cv-h24*(-3.*p(2,2,ip1)-10.0d0*p(2,2,i)+p(2,2,im1))-

     -       h3p12*(p(2,1,i)*ciclcv+p(2,2,i)*cicl)


      f3v  = f3v-h3p12*(p(2,1,i)*ficlv+p(2,2,i)*ficl)

c

c for system with first derivatives.  (end corrections 2)
c

c
c--------------------------------- i=4
c      print *,'c--------------------------------- i=4'
c

corr for general case

	i  =4
      im1=i-1
      ip1=i+1

      a4   = 1.0d0+hd12*f(1,1,im1)
      b4   = h5d6*f(1,1,i)-2.0d0
      c4   = 1.0d0+hd12*f(1,1,ip1)
      a4c  = hd12*f(1,2,im1)
      b4c  = h5d6*f(1,2,i)
      c4c  = hd12*f(1,2,ip1)
      f4   = uk(1,i)*hh+hd12*(uk(1,ip1)-2.*uk(1,i)+uk(1,im1))
c
      a4v  = 1.0d0+hd12*f(2,1,im1)
      b4v  = h5d6*f(2,1,i)-2.
      c4v  = 1.0d0+hd12*f(2,1,ip1)
      a4cv = hd12*f(2,2,im1)
      b4cv = h5d6*f(2,2,i)
      c4cv = hd12*f(2,2,ip1)
      f4v  = uk(2,i)*hh+hd12*(uk(2,ip1)-2.0d0*uk(2,i)+uk(2,im1))

c
c for system with first derivatives.  (begin corrections 3, formulas 22,16)
c
      aicl  = f(1,1,im1)-0.50d0*(p(1,1,ip1)+3.0d0*p(1,1,im1))/h
      bicl  = 2.0d0*(p(1,1,ip1)+p(1,1,im1))/h
      cicl  = -f(1,1,ip1)-0.50d0*(3.0d0*p(1,1,ip1)+p(1,1,im1))/h
      aiclc = f(1,2,im1)-0.50d0*(p(1,2,ip1)+3.0d0*p(1,2,im1))/h
      biclc = 2.0d0*(p(1,2,ip1)+p(1,2,im1))/h
      ciclc = -f(1,2,ip1)-0.50d0*(3.*p(1,2,ip1)+p(1,2,im1))/h
      ficl  = uk(1,ip1)-uk(1,im1)
c
      aiclv = f(2,1,im1)-0.50d0*(p(2,1,ip1)+3.0d0*p(2,1,im1))/h
      biclv = 2.0d0*(p(2,1,ip1)+p(2,1,im1))/h
      ciclv = -f(2,1,ip1)-0.50d0*(3.*p(2,1,ip1)+p(2,1,im1))/h
      aiclcv= f(2,2,im1)-0.50d0*(p(2,2,ip1)+3.*p(2,2,im1))/h
      biclcv= 2.0d0*(p(2,2,ip1)+p(2,2,im1))/h
      ciclcv= -f(2,2,ip1)-0.50d0*(3.*p(2,2,ip1)+p(2,2,im1))/h
      ficlv=  uk(2,ip1)-uk(2,im1)


      a4   = a4-h24*(-p(1,1,ip1)+10.0d0*p(1,1,i)+3.0d0*p(1,1,im1))-
     -       h3p12*(p(1,1,i)*aicl+p(1,2,i)*aiclcv)
      b4   = b4-h6*(p(1,1,ip1)-p(1,1,im1))-
     -       h3p12*(p(1,1,i)*bicl+p(1,2,i)*biclcv)
      c4   = c4-h24*(-3.*p(1,1,ip1)-10.*p(1,1,i)+p(1,1,im1))-
     -       h3p12*(p(1,1,i)*cicl+p(1,2,i)*ciclcv)
      a4c  = a4c-h24*(-p(1,2,ip1)+10.*p(1,2,i)+3.*p(1,2,im1))-
     -       h3p12*(p(1,1,i)*aiclc+p(1,2,i)*aiclv)
      b4c  = b4c-h6*(p(1,2,ip1)-p(1,2,im1))-
     -       h3p12*(p(1,1,i)*biclc+p(1,2,i)*biclv)
      c4c  = c4c-h24*(-3.*p(1,2,ip1)-10.*p(1,2,i)+p(1,2,im1))-
     -       h3p12*(p(1,1,i)*ciclc+p(1,2,i)*ciclv)


      f4   = f4-h3p12*(p(1,1,i)*ficl+p(1,2,i)*ficlv)

c
      a4v  = a4v-h24*(-p(2,1,ip1)+10.*p(2,1,i)+3.*p(2,1,im1))-
     -       h3p12*(p(2,1,i)*aiclv+p(2,2,i)*aiclc)
      b4v  = b4v-h6*(p(2,1,ip1)-p(2,1,im1))-
     -       h3p12*(p(2,1,i)*biclv+p(2,2,i)*biclc)
      c4v  = c4v-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-
c     -       h3p12*(p(2,1,i)*ciclc+p(2,2,i)*ciclc)
     -       h3p12*(p(2,1,i)*ciclv+p(2,2,i)*ciclc)
      a4cv = a4cv-h24*(-p(2,2,ip1)+10.*p(2,2,i)+3.*p(2,2,im1))-
     -       h3p12*(p(2,1,i)*aiclcv+p(2,2,i)*aicl)
      b4cv = b4cv-h6*(p(2,2,ip1)-p(2,2,im1))-
     -       h3p12*(p(2,1,i)*biclcv+p(2,2,i)*bicl)

corr 17.07.97
c      c4cv = c4cv-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-
      c4cv = c4cv-h24*(-3.0d0*p(2,2,ip1)-10.0d0*p(2,2,i)+p(2,2,im1))-

     -       h3p12*(p(2,1,i)*ciclcv+p(2,2,i)*cicl)

      f4v  = f4v-h3p12*(p(2,1,i)*ficlv+p(2,2,i)*ficl)

c
c for system with first derivatives.  (end corrections 3)
c

c formulas (42)
      zz2   = b1 *a2-a1*b2
      zz3   = c1 *a2-a1*c2
	zz4   = dd1*a2-a1*dd2
	zz5   = e1 *a2-a1*e2
	zz6   =       -a1*b2c
	zz7   =       -a1*c2c
	zz8   =       -a1*dd2c
	zz9   =       -a1*e2c
	qq3   = a2*uk1h-a1*uk2h

      zv2   = b1v *a2v-a1v*b2v
      zv3   = c1v *a2v-a1v*c2v
	zv4   = dd1v*a2v-a1v*dd2v
	zv5   = e1v *a2v-a1v*e2v
	zv6   =       -a1v*b2cv
	zv7   =       -a1v*c2cv
	zv8   =       -a1v*dd2cv
	zv9   =       -a1v*e2cv
	qv3   = a2v*uk1hv-a1v*uk2hv



	ww3 = zv6*zz3-zz2*zv7
	ww4 = zv6*zz4-zz2*zv8
	ww5 = zv6*zz5-zz2*zv9
	ww6 = zv6*zz6-zz2*zv2
	ww7 = zv6*zz7-zz2*zv3
	ww8 = zv6*zz8-zz2*zv4
	ww9 = zv6*zz9-zz2*zv5
	qq4 = zv6*qq3-zz2*qv3


	vv3 = a3*zz3-zz2*b3
	vv4 = a3*zz4-zz2*c3
	vv5 = a3*zz5
	vv6 = a3*zz6-zz2*a3c
	vv7 = a3*zz7-zz2*b3c
	vv8 = a3*zz8-zz2*c3c
	vv9 = a3*zz9
	qq5 = a3*qq3-zz2*f3


	xx3 = a3cv*zz3-zz2*b3cv
	xx4 = a3cv*zz4-zz2*c3cv
	xx5 = a3cv*zz5
	xx6 = a3cv*zz6-zz2*a3v
	xx7 = a3cv*zz7-zz2*b3v
	xx8 = a3cv*zz8-zz2*c3v
	xx9 = a3cv*zz9
	qq6 = a3cv*qq3-zz2*f3v


	ss1 = vv6*ww3-ww6*vv3
	ss2 = vv6*ww4-ww6*vv4
	ss3 = vv6*ww5-ww6*vv5
	ss4 = vv6*ww7-ww6*vv7
	ss5 = vv6*ww8-ww6*vv8
	ss6 = vv6*ww9-ww6*vv9
	qq7 = vv6*qq4-ww6*qq5


	sv1 = xx6*ww3-ww6*xx3
	sv2 = xx6*ww4-ww6*xx4
	sv3 = xx6*ww5-ww6*xx5
	sv4 = xx6*ww7-ww6*xx7
	sv5 = xx6*ww8-ww6*xx8
	sv6 = xx6*ww9-ww6*xx9
	qv7 = xx6*qq4-ww6*qq6


	al1 = sv1*ss2-ss1*sv2
	al2 = sv1*ss3-ss1*sv3
	al3 = sv1*ss4-ss1*sv4
	al4 = sv1*ss5-ss1*sv5
	al5 = sv1*ss6-ss1*sv6
	qq8 = sv1*qq7-ss1*qv7


	be1 = a4*ss2-ss1*b4
	be2 = a4*ss3-ss1*c4
	be3 = a4*ss4-ss1*a4c
	be4 = a4*ss5-ss1*b4c
	be5 = a4*ss6-ss1*c4c
	qq9 = a4*qq7-ss1*f4

	ga1 = a4cv*ss2-ss1*b4cv
	ga2 = a4cv*ss3-ss1*c4cv
	ga3 = a4cv*ss4-ss1*a4v
	ga4 = a4cv*ss5-ss1*b4v
	ga5 = a4cv*ss6-ss1*c4v
	q10 = a4cv*qq7-ss1*f4v


	slm1 = be3*al1-al3*be1
	slm2 = be3*al2-al3*be2
	slm3 = be3*al4-al3*be4
	slm4 = be3*al5-al3*be5
	q11  = be3*qq8-al3*qq9


	slmv1 = ga3*al1-al3*ga1
	slmv2 = ga3*al2-al3*ga2
	slmv3 = ga3*al4-al3*ga4
	slmv4 = ga3*al5-al3*ga5
	q11v  = ga3*qq8-al3*q10


	pp1 = slmv1*slm2-slm1*slmv2
	pp2 = slmv1*slm3-slm1*slmv3
	pp3 = slmv1*slm4-slm1*slmv4
	pp4 = slmv1*q11 -slm1*q11v


	pv1 = slmv3*slm1-slm3*slmv1
	pv2 = slmv3*slm2-slm3*slmv2
	pv3 = slmv3*slm4-slm3*slmv4
	pv4 = slmv3*q11 -slm3*q11v

c
      am(1,1,4) = -pv2/pv1
      am(1,2,4) = -pv3/pv1
      bm(1,4)   = +pv4/pv1
      am(2,2,4) = -pp1/pp2
      am(2,1,4) = -pp3/pp2
      bm(2,4)   = +pp4/pp2



c----------------------------------- from i=5 to i=n-2
c      print *,'c----------------------------------- from i=5 to i=n-2'
c
      do 10 i=5,n2
      im1=i-1
      ip1=i+1
c
      ai   = 1.+hd12*f(1,1,im1)
      bi   = h5d6*f(1,1,i)-2.0d0
      ci   = 1.+hd12*f(1,1,ip1)
      aic  = hd12*f(1,2,im1)
      bic  = h5d6*f(1,2,i)
      cic  = hd12*f(1,2,ip1)
      fi   = uk(1,i)*hh+hd12*(uk(1,ip1)-2.*uk(1,i)+uk(1,im1))
c
      aiv  = 1.0d0+hd12*f(2,1,im1)
      biv  = h5d6*f(2,1,i)-2.0d0
      civ  = 1.0d0+hd12*f(2,1,ip1)
      aicv = hd12*f(2,2,im1)
      bicv = h5d6*f(2,2,i)
      cicv = hd12*f(2,2,ip1)
      fiv  = uk(2,i)*hh+hd12*(uk(2,ip1)-2.0d0*uk(2,i)+uk(2,im1))
c
c for system with first derivatives.  (begin corrections 4, formulas 22,16)
c
      aicl  =  f(1,1,im1)-0.50d0*(p(1,1,ip1)+3.0d0*p(1,1,im1))/h
      bicl  =  2.0d0*(p(1,1,ip1)+p(1,1,im1))/h
      cicl  = -f(1,1,ip1)-0.50d0*(3.*p(1,1,ip1)+p(1,1,im1))/h

corr 17.07.97
c      aiclc =  f(1,2,im1)-0.5*(p(1,2,ip1)+3.*p(1,1,im1))/h

      aiclc =  f(1,2,im1)-0.50d0*(p(1,2,ip1)+3.0d0*p(1,2,im1))/h
      biclc =  2.0d0*(p(1,2,ip1)+p(1,2,im1))/h
      ciclc = -f(1,2,ip1)-0.50d0*(3.*p(1,2,ip1)+p(1,2,im1))/h
      ficl  =  uk(1,ip1)-uk(1,im1)
c
      aiclv =  f(2,1,im1)-0.50d0*(p(2,1,ip1)+3.0d0*p(2,1,im1))/h
      biclv =  2.0d0*(p(2,1,ip1)+p(2,1,im1))/h
      ciclv = -f(2,1,ip1)-0.50d0*(3.*p(2,1,ip1)+p(2,1,im1))/h

corr 17.07.97
c      aiclcv=  f(2,2,im1)-0.5*(p(2,2,ip1)+3.*p(2,1,im1))/h

      aiclcv=  f(2,2,im1)-0.50d0*(p(2,2,ip1)+3.0d0*p(2,2,im1))/h
      biclcv=  2.0d0*(p(2,2,ip1)+p(2,2,im1))/h
      ciclcv= -f(2,2,ip1)-0.50d0*(3.*p(2,2,ip1)+p(2,2,im1))/h
      ficlv=   uk(2,ip1)-uk(2,im1)



      ai   = ai-h24*(-p(1,1,ip1)+10.0d0*p(1,1,i)+3.*p(1,1,im1))-
     -       h3p12*(p(1,1,i)*aicl+p(1,2,i)*aiclcv)
      bi   = bi-h6*(p(1,1,ip1)-p(1,1,im1))-
     -       h3p12*(p(1,1,i)*bicl+p(1,2,i)*biclcv)
      ci   = ci-h24*(-3.*p(1,1,ip1)-10.0d0*p(1,1,i)+p(1,1,im1))-
     -       h3p12*(p(1,1,i)*cicl+p(1,2,i)*ciclcv)
      aic  = aic-h24*(-p(1,2,ip1)+10.0d0*p(1,2,i)+3.*p(1,2,im1))-
     -       h3p12*(p(1,1,i)*aiclc+p(1,2,i)*aiclv)
      bic  = bic-h6*(p(1,2,ip1)-p(1,2,im1))-
     -       h3p12*(p(1,1,i)*biclc+p(1,2,i)*biclv)

corr 17.07.97
c      cc   = cic-h24*(-3.*p(1,2,ip1)-10.*p(1,2,i)+p(1,2,im1))-
      cic  = cic-h24*(-3.*p(1,2,ip1)-10.0d0*p(1,2,i)+p(1,2,im1))-

     -       h3p12*(p(1,1,i)*ciclc+p(1,2,i)*ciclv)

      fi   = fi-h3p12*(p(1,1,i)*ficl+p(1,2,i)*ficlv)

c
      aiv  = aiv-h24*(-p(2,1,ip1)+10.*p(2,1,i)+3.*p(2,1,im1))-
     -       h3p12*(p(2,1,i)*aiclv+p(2,2,i)*aiclc)
      biv  = biv-h6*(p(2,1,ip1)-p(2,1,im1))-
     -       h3p12*(p(2,1,i)*biclv+p(2,2,i)*biclc)
      civ  = civ-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-
c     -       h3p12*(p(2,1,i)*ciclc+p(2,2,i)*ciclc)
     -       h3p12*(p(2,1,i)*ciclv+p(2,2,i)*ciclc)
      aicv = aicv-h24*(-p(2,2,ip1)+10.*p(2,2,i)+3.*p(2,2,im1))-
     -       h3p12*(p(2,1,i)*aiclcv+p(2,2,i)*aicl)
      bicv = bicv-h6*(p(2,2,ip1)-p(2,2,im1))-
     -       h3p12*(p(2,1,i)*biclcv+p(2,2,i)*bicl)
corr 15.07.97
c      cicv = cicv-h24*(-3.*p(2,1,ip1)-10.*p(2,1,i)+p(2,1,im1))-

      cicv = cicv-h24*(-3.*p(2,2,ip1)-10.*p(2,2,i)+p(2,2,im1))-
     -       h3p12*(p(2,1,i)*ciclcv+p(2,2,i)*cicl)


      fiv  = fiv-h3p12*(p(2,1,i)*ficlv+p(2,2,i)*ficl)

c
c for system with first derivatives.  (end corrections 4)
c

c formulas (57)-(61)
      zz   = ai *am(1,1,im1)+aic *am(2,2,im1)+bi
      ff   = ai *am(1,2,im1)+aic *am(2,1,im1)+bic
      zzv  = aiv*am(2,1,im1)+aicv*am(1,2,im1)+biv
      ffv  = aiv*am(2,2,im1)+aicv*am(1,1,im1)+bicv
      fih  = fi -ai *bm(1,im1)-aic *bm(2,im1)
      fihv = fiv-aiv*bm(2,im1)-aicv*bm(1,im1)
c
      znm  =  ffv/zz
      si1  = -znm* ff+ zzv
      si2  = -znm* ci+ cicv
      si3  = -znm*cic+ civ
      si4  = -znm*fih+ fihv
      znmc =  ff/zzv
      si1c = -znmc*ffv +zz
      si2c = -znmc*cicv+ci
      si3c = -znmc*civ +cic
      si4c = -znmc*fihv+fih
c
      bm(1,i)   =  si4c/si1c
      am(1,1,i) = -si2c/si1c
      am(1,2,i) = -si3c/si1c
      bm(2,i)   =  si4/si1
      am(2,1,i) = -si3/si1
      am(2,2,i) = -si2/si1
c
c      if(i.ge.n5)write(20,*)' i=',i,' am(1,1,i)=',am(1,1,i),
c     ,' am(1,2,i)=',am(1,2,i),' am(2,1,i)=',am(2,1,i),' am(2,2,i)=',
c     ,am(2,2,i)
c      if(i.ge.n5)write(20,*)' i=',i,' bm(1,i)=',bm(1,i),' bm(2,i)=',
c     ,bm(2,i)
c
c      if(i.lt.10)write(20,*)' i=',i,' am(1,1,i)=',am(1,1,i),
c     ,' am(1,2,i)=',am(1,2,i),' am(2,1,i)=',am(2,1,i),' am(2,2,i)=',
c     ,am(2,2,i)
c      if(i.lt.10)write(20,*)' i=',i,' bm(1,i)=',bm(1,i),' bm(2,i)=',
c     ,bm(2,i)

  10  continue
c

c---------------------------------------- the right system
c

c--------------------------------- i=n
c      print *,'c--------------------------------- i=n'
      an   =  25.0d0*d2(1)+h12*f2(1)
      bn   = -48.0d0*d2(1)
      cn   =  36.0d0*d2(1)
      dn   = -16.0d0*d2(1)
      en   =  3.0d0*d2(1)
      ukn  =  g2(1)*h12
c
      anv  =  25.0d0*d2(2)+h12*f2(2)
      bnv  = -48.0d0*d2(2)
      cnv  =  36.0d0*d2(2)
      dnv  = -16.0d0*d2(2)
      env  =  3.0d0*d2(2)
      uknv =  g2(2)*h12

c
c--------------------------------- i=n-1
c
corr for general case
	  i  = n1
	im1  = i-1
	ip1  = i+1

c formulas (26)
      an1  =  1.
      bn1  = -2.0d0+h7d6*f(1,1,n1)
      cn1  =  1.0d0-h5d12*f(1,1,n2)
      dn1  =  hd3*f(1,1,n3)
      en1  = -hd12*f(1,1,n4)
      bnc1 =  h7d6*f(1,2,n1)
      cnc1 = -h5d12*f(1,2,n2)
      dnc1 =  hd3*f(1,2,n3)
      enc1 = -hd12*f(1,2,n4)
      uknh1=  uk(1,n1)*h7d6-h5d12*uk(1,n2)+hd3*uk(1,n3)-hd12*uk(1,n4)
c


      anv1 =  1.0d0
      bnv1 = -2.0d0+h7d6*f(2,1,n1)
      cnv1 =  1.0d0-h5d12*f(2,1,n2)
      dnv1 =  hd3*f(2,1,n3)
      env1 = -hd12*f(2,1,n4)
      bncv1=  h7d6*f(2,2,n1)
      cncv1= -h5d12*f(2,2,n2)
      dncv1=  hd3*f(2,2,n3)
      encv1= -hd12*f(2,2,n4)
      uknhv1 = uk(2,n1)*h7d6-h5d12*uk(2,n2)+hd3*uk(2,n3)-hd12*uk(2,n4)

c      write(20,*)'uknh1=',uknh1,' uk(1,n1)=',uk(1,n1),' uk(1,n2)=',
c     ,uk(1,n2),' uk(1,n3)=',uk(1,n3),' uk(1,n4)=',uk(1,n4)
c      write(20,*)'uknhv1=',uknhv1,' uk(2,n1)=',uk(2,n1),' uk(2,n2)=',
c     ,uk(2,n2),' uk(2,n3)=',uk(2,n3),' uk(2,n4)=',uk(2,n4)
corr for general case
	anc1  = 0.
	ancv1 = 0.

c
c for system with first derivatives.  (begin corrections 5, formulas 20,26)
c


	r    =  0.50d0*p(1,1,n3)/h
	s    = -f(1,1,n3)-2.0d0*p(1,1,n2)/h
	u    = -0.50d0*p(1,1,n3)/h+4.0d0*f(1,1,n2)+1.50d0*p(1,1,n1)/h
	v    =  2.0d0*p(1,1,n2)/h-3.0d0*f(1,1,n1)
	w    = -1.50d0*p(1,1,n1)/h

c	rc   =  0.5*p(1,2,n3)
	rc   =  0.50d0*p(1,2,n3)/h
	sc   = -f(1,2,n3)-2.0d0*p(1,2,n2)/h
	uc   = -0.50d0*p(1,2,n3)/h+4.0d0*f(1,2,n2)+1.50d0*p(1,2,n1)/h
	vc   =  2.0d0*p(1,2,n2)/h-3.0d0*f(1,2,n1)
	wc   = -1.50d0*p(1,2,n1)/h
	t    =  uk(1,n3)-4.0d0*uk(1,n2)+3.0d0*uk(1,n1)

c	rv   =  0.5*p(2,1,n3)
	rv   =  0.50d0*p(2,1,n3)/h
	sv   = -f(2,1,n3)-2.0d0*p(2,1,n2)/h
	uv   = -0.50d0*p(2,1,n3)/h+4.0d0*f(2,1,n2)+1.50d0*p(2,1,n1)/h
	vv   =  2.0d0*p(2,1,n2)/h-3.0d0*f(2,1,n1)
	wv   = -1.50d0*p(2,1,n1)/h

c	rcv  =  0.5*p(2,2,n3)
	rcv  =  0.50d0*p(2,2,n3)/h
	scv  = -f(2,2,n3)-2.0d0*p(2,2,n2)/h
	ucv  = -0.50d0*p(2,2,n3)/h+4.0d0*f(2,2,n2)+1.50d0*p(2,2,n1)/h
	vcv  =  2.0d0*p(2,2,n2)/h-3.0d0*f(2,2,n1)
	wcv  = -1.50d0*p(2,2,n1)/h
	tv   =  uk(2,n3)-4.0d0*uk(2,n2)+3.0d0*uk(2,n1)


	an1   = an1+h712*p(1,1,n1)-h3p12*(p(1,1,n1)*w+p(1,2,n1)*wcv)
	bn1   = bn1-h524*p(1,1,n2)-h3p12*(p(1,1,n1)*v+p(1,2,n1)*vcv)
	cn1   = cn1-h24*(-p(1,1,n4)-4.0d0*p(1,1,n3)+14.*p(1,1,n1))-
     -          h3p12*(p(1,1,n1)*u+p(1,2,n1)*ucv)
	dn1   = dn1-h24*(4.*p(1,1,n4)-5.0d0*p(1,1,n2))-
     -          h3p12*(p(1,1,n1)*s+p(1,2,n1)*scv)
	en1   = en1+h8*p(1,1,n4)-
     -          h3p12*(p(1,1,n1)*r+p(1,2,n1)*rcv)
	anc1  = anc1+h712*p(1,2,n1)-
     -          h3p12*(p(1,1,n1)*wc+p(1,2,n1)*wv)
	bnc1  = bnc1-h524*p(1,2,n2)-
     -          h3p12*(p(1,1,n1)*vc+p(1,2,n1)*vv)
	cnc1  = cnc1-h24*(-p(1,2,n4)-4.0d0*p(1,2,n3)+14.*p(1,2,n1))-
     -          h3p12*(p(1,1,n1)*uc+p(1,2,n1)*uv)
	dnc1  = dnc1-h24*(4.*p(1,2,n1)-5.0d0*p(1,2,n2))-
     -          h3p12*(p(1,1,n1)*sc+p(1,2,n1)*sv)
	enc1  = enc1+h8*p(1,2,n4)-
     -          h3p12*(p(1,1,n1)*rc+p(1,2,n1)*rv)


      uknh1   = uknh1+h3p12*(p(1,1,n1)*t+p(1,2,n1)*tv)


c
	anv1  = anv1+h712*p(2,1,n1)-h3p12*(p(2,1,n1)*wv+p(2,2,n1)*wc)
	bnv1  = bnv1-h524*p(2,1,n2)-h3p12*(p(2,1,n1)*vv+p(2,2,n1)*vc)
	cnv1  = cnv1-h24*(-p(2,1,n4)-4.0d0*p(2,1,n3)+14.0d0*p(2,1,n1))-
     -          h3p12*(p(2,1,n1)*uv+p(2,2,n1)*uc)
	dnv1  = dnv1-h24*(4.*p(2,1,n4)-5.0d0*p(2,1,n1))-
     -          h3p12*(p(2,1,n1)*sv+p(2,2,n1)*sc)
	env1  = env1+h8*p(2,1,n4)-
     -          h3p12*(p(2,1,n1)*rv+p(2,2,n1)*rc)
	ancv1 = ancv1+h712*p(2,2,n1)-
     -          h3p12*(p(2,1,n1)*wcv+p(2,2,n1)*w)
	bncv1 = bncv1-h524*p(2,2,n2)-
     -          h3p12*(p(2,1,n1)*vcv+p(2,2,n1)*v)
	cncv1 = cncv1-h24*(-p(2,2,n4)-4.0d0*p(2,2,n3)+14.0d0*p(2,2,n1))-
     -          h3p12*(p(2,1,n1)*ucv+p(2,2,n1)*u)
	dncv1 = dncv1-h24*(4.*p(2,2,n1)-5.0d0*p(2,2,n2))-
     -          h3p12*(p(2,1,n1)*scv+p(2,2,n1)*s)
	encv1 = encv1+h8*p(2,2,n4)-
     -          h3p12*(p(2,1,n1)*rcv+p(2,2,n1)*r)


      uknhv1  = uknhv1+h3p12*(p(2,1,n1)*tv+p(2,2,n1)*t)

c      write(20,*)' t=',t,' tv=',tv
c      write(20,*)' p(1,1)=',p(1,1,2),' p(1,2)=',p(1,2,2)
c      write(20,*)'uknh1=',uknh1,' uk(1,n1)=',uk(1,n1),' uk(1,n2)=',
c     ,uk(1,n2),' uk(1,n3)=',uk(1,n3),' uk(1,n4)=',uk(1,n4)
c      write(20,*)' p(2,1)=',p(2,1,2),' p(2,2)=',p(2,2,2)
c
c      write(20,*)'uknhv1=',uknhv1,' uk(2,n1)=',uk(2,n1),' uk(2,n2)=',
c     ,uk(2,n2),' uk(2,n3)=',uk(2,n3),' uk(2,n4)=',uk(2,n4)

c
c for system with first derivatives.  (end corrections 5)
c
c
      do 33 i= 1,10
      do 33 j= 1,11
   33 e(i,j) = 0.
c
      e(1,1) = an
      e(1,2) = bn
      e(1,3) = cn
      e(1,4) = dn
      e(1,5) = en
      e(1,11)= ukn
c
      e(2,6) = anv
      e(2,7) = bnv
      e(2,8) = cnv
      e(2,9) = dnv
      e(2,10)= env
      e(2,11)= uknv
c
      e(3,1) = an1
      e(3,2) = bn1
      e(3,3) = cn1
      e(3,4) = dn1
      e(3,5) = en1
      e(3,7) = bnc1
      e(3,8) = cnc1
      e(3,9) = dnc1
      e(3,10)= enc1
      e(3,11)= uknh1
c
      e(4,2) = bncv1
      e(4,3) = cncv1
      e(4,4) = dncv1
      e(4,5) = encv1
      e(4,6) = anv1
      e(4,7) = bnv1
      e(4,8) = cnv1
      e(4,9) = dnv1
      e(4,10)= env1
      e(4,11)= uknhv1

c
c for system with first derivatives.  (begin corrections 6)
c

      e(3,6) = anc1
      e(4,1) = ancv1


c
c for system with first derivatives.  (end corrections 6)
c
c
      e(5,2)   = am(1,1,n2)
      e(5,3)   = -1.0d0
      e(5,7)   = am(1,2,n2)
      e(5,11)  = -bm(1,n2)
c
      e(6,2)   = am(2,2,n2)
      e(6,7)   = am(2,1,n2)
      e(6,8)   = -1.0d0
      e(6,11)  = -bm(2,n2)
c
      e(7,3)   =  am(1,1,n3)
      e(7,4)   = -1.0d0
      e(7,8)   =  am(1,2,n3)
      e(7,11)  = -bm(1,n3)
c
      e(8,3)   =  am(2,2,n3)
      e(8,8)   =  am(2,1,n3)
      e(8,9)   = -1.0d0
      e(8,11)  = -bm(2,n3)
c
      e(9,4)   =  am(1,1,n4)
      e(9,5)   = -1.0d0
      e(9,9)   =  am(1,2,n4)
      e(9,11)  = -bm(1,n4)
c
      e(10,4)  =  am(2,2,n4)
      e(10,9)  =  am(2,1,n4)
      e(10,10) = -1.0d0
      e(10,11) = -bm(2,n4)
ccc
      do j=1,12
      do i=1,10
c      write(20,*)' i=',i,' j=',j,' e(i,j)=',e(i,j)
      end do
      end do

      call matin2(e,10,10,mdim,1,index,nerror,determ)
ccc
      y(1,n) = e(1,11)
      y(1,n1)= e(2,11)
      y(1,n2)= e(3,11)
      y(1,n3)= e(4,11)
      y(1,n4)= e(5,11)
      y(2,n) = e(6,11)
      y(2,n1)= e(7,11)
      y(2,n2)= e(8,11)
      y(2,n3)= e(9,11)
      y(2,n4)= e(10,11)
c
c------------------------------------ the reverse 'progonka'
c
  18  continue
      do 3 i = n5,4,-1
      ip1    = i+1
      y(1,i) = bm(1,i)+am(1,1,i)*y(1,ip1)+am(1,2,i)*y(2,ip1)
      y(2,i) = bm(2,i)+am(2,1,i)*y(2,ip1)+am(2,2,i)*y(1,ip1)
  3   continue

      y(2,3) = (qq8-al5*y(2,5)-al4*y(2,4)-al2*y(1,5)-al1*y(1,4))/al3
	y(1,3) = (qq7-ss6*y(2,5)-ss5*y(2,4)-ss4*y(2,3)-ss3*y(1,5)-
     ^		 ss2*y(1,4))/ss1
      y(2,2) = (qq4-ww3*y(1,3)-ww4*y(1,4)-ww5*y(1,5)-ww7*y(2,3)-
     ^		 ww8*y(2,4)-ww9*y(2,5))/ww6
	y(1,2) = (qq3-zz3*y(1,3)-zz4*y(1,4)-zz5*y(1,5)-zz6*y(2,2)-
     ^		 zz7*y(2,3)-zz8*y(2,4)-zz9*y(2,5))/zz2
      y(1,1) = (uk1h -b1*y(1,2) -c1 *y(1,3)-dd1 *y(1,4)-e1 *y(1,5))/a1
      y(2,1) = (uk1hv-b1v*y(2,2)-c1v*y(2,3)-dd1v*y(2,4)-e1v*y(2,5))/a1v
c
      return
      end
c*
c *****************************************************************************


c*
C***********************************************************
c*
      SUBROUTINE TAUKA(NTAU,TP,D0,D1,TAU0,TAU)
      IMPLICIT REAL*8(A-H,O-Z)

      IF(NTAU.EQ.1)then 
	TAU = TAU0
	return
	endif
	      
      IF(NTAU.EQ.2)then
      IF(D1.LT.D0)TAU = MIN(1.d0,2.d0*TP)
      IF(D1.GE.D0)TAU = MAX(TAU0,TP/2.d0)
      return
	endif

      IF(NTAU.EQ.3)then
      IF(D1.LT.D0)TAU = MIN(1.d0,TP*(D0/D1))
      IF(D1.GE.D0)TAU = MAX(TAU0,TP*(D0/D1))
      return
	endif

      RETURN
      END
c
C*
C************************************************
C*
      SUBROUTINE D1T5(NR,R,P,DP)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION R(NR),P(NR),DP(NR)
      N=NR-2
      DO 1 I=3,N
      R1=R(I-2)
      R2=R(I-1)
      R3=R(I)
      R4=R(I+1)
      R5=R(I+2)
      FH1=P(I-2)/((R1-R2)*(R1-R3)*(R1-R4)*(R1-R5))
      FH2=P(I-1)/((R2-R1)*(R2-R3)*(R2-R4)*(R2-R5))
      FH3=P(I)/((R3-R1)*(R3-R2)*(R3-R4)*(R3-R5))
      FH4=P(I+1)/((R4-R1)*(R4-R2)*(R4-R3)*(R4-R5))
      FH5=P(I+2)/((R5-R1)*(R5-R2)*(R5-R3)*(R5-R4))
      DP(I)=FH1*(R3-R2)*(R3-R4)*(R3-R5)+FH2*(R3-R1)*(R3-R4)*(R3-R5)+
     +FH3*((R3-R2)*(R3-R4)*(R3-R5)+(R3-R1)*(R3-R4)*(R3-R5)+
     +(R3-R1)*(R3-R2)*(R3-R5)+(R3-R1)*(R3-R2)*(R3-R4))+
     +FH4*(R3-R1)*(R3-R2)*(R3-R5)+FH5*(R3-R1)*(R3-R2)*(R3-R4)
      IF(I.GT.3)GO TO 1
      DP(1)=FH1*((R1-R3)*(R1-R5)*(R1-R4)+(R1-R2)*(R1-R4)*(R1-R5)+
     +(R1-R2)*(R1-R3)*(R1-R5)+(R1-R2)*(R1-R3)*(R1-R4))+
     +FH2*(R1-R3)*(R1-R4)*(R1-R5)+FH3*(R1-R2)*(R1-R4)*(R1-R5)+
     +FH4*(R1-R2)*(R1-R3)*(R1-R5)+FH5*(R1-R2)*(R1-R3)*(R1-R4)
      DP(2)=FH1*(R2-R3)*(R2-R4)*(R2-R5)+FH2*((R2-R3)*(R2-R4)*(R2-R5)+
     +(R2-R1)*(R2-R4)*(R2-R5)+(R2-R1)*(R2-R3)*(R2-R5)+(R2-R1)*(R2-R3)*
     *(R2-R4))+FH3*(R2-R1)*(R2-R4)*(R2-R5)+FH4*(R2-R1)*(R2-R3)*(R2-R5)+
     +FH5*(R2-R1)*(R2-R3)*(R2-R4)
    1 CONTINUE
      DP(NR-1)=FH1*(R4-R2)*(R4-R3)*(R4-R5)+
     +FH2*(R4-R1)*(R4-R3)*(R4-R5)+FH3*(R4-R1)*(R4-R2)*(R4-R5)+
     +FH4*((R4-R2)*(R4-R3)*(R4-R5)+(R4-R1)*(R4-R3)*(R4-R5)+
     +(R4-R1)*(R4-R2)*(R4-R5)+(R4-R1)*(R4-R2)*(R4-R3))
     ++FH5*(R4-R1)*(R4-R2)*(R4-R3)
      DP(NR)=FH1*(R5-R2)*(R5-R3)*(R5-R4)+FH2*(R5-R1)*(R5-R3)*(R5-R4)+
     +FH3*(R5-R1)*(R5-R2)*(R5-R4)+FH4*(R5-R1)*(R5-R2)*(R5-R3)+
     +FH5*((R5-R2)*(R5-R3)*(R5-R4)+(R5-R1)*(R5-R3)*(R5-R4)+
     +(R5-R1)*(R5-R2)*(R5-R4)+(R5-R3)*(R5-R1)*(R5-R2))
      RETURN
      END
c*
c***********************************************************
c*
      subroutine simpson(n,h,f,s)
      implicit real*8 (a-h,o-z)
      dimension f(n)
      s   = f(1)+4.*f(2)
      nm2 = n-2
      do i= 3,nm2,2
      s   = s+f(i)*2.+f(i+1)*4.
      end do
      s   = (s+f(n))*h/3.
      return
      END

c*
C***************************************************************
C*
      SUBROUTINE  MATIN2 (A,IDIM1,N1,IDIM2,N2,INDEX,NERROR,DETERM)
c     implicit real*8 (a-h,o-z)
C        MATRIX INVERSION WITH ACCOMPANYING SOLUTION OF LINEAR EQUATIONS
      DOUBLE PRECISION A,DETERM,DETER,PIVOT,SWAP
      DIMENSION A(*),INDEX(*)
      DETER=1.0D0
      N=N1
      IEMAT=N+N2
      IDIM=IDIM1
      NMIN1=N-1
C        THE ROUTINE DOES ITS OWN EVALUATION FOR DOUBLE SUBSCRIPTING OF
C        ARRAY A.
      IPIVC=1-IDIM
C        MAIN LOOP TO INVERT THE MATRIX
      DO 11 MAIN=1,N
      PIVOT=0.0D0
      IPIVC=IPIVC+IDIM
C        SEARCH FOR NEXT PIVOT IN COLUMN MAIN.
      IPIVC1=IPIVC+MAIN-1
      IPIVC2=IPIVC +NMIN1
      DO 2 I1=IPIVC1,IPIVC2
      IF(DABS(A(I1))-DABS(PIVOT)) 2,2,1
    1 PIVOT=A(I1)
      LPIV=I1
    2 CONTINUE
C        IS PIVOT DIFFERENT FROM ZERO
      IF(PIVOT) 3,15,3
C        GET THE PIVOT-LINE INDICATOR AND SWAP LINES IF NECESSARY
    3 ICOL=LPIV-IPIVC+1
      INDEX(MAIN)=ICOL
      IF(ICOL-MAIN) 6,6,4
C        COMPLEMENT THE DETERMINANT
    4 DETER=-DETER
C        POINTER TO LINE PIVOT FOUND
      ICOL=ICOL-IDIM
C        POINTER TO EXACT PIVOT LINE
      I3=MAIN-IDIM
      DO 5 I=1,IEMAT
      ICOL=ICOL+IDIM
      I3=I3+IDIM
      SWAP=A(I3)
      A(I3)=A(ICOL)
    5 A(ICOL)=SWAP
C        COMPUTE DETERMINANT
    6 DETER=DETER*PIVOT
      PIVOT=1./PIVOT
C        TRANSFORM PIVOT COLUMN
      I3=IPIVC+NMIN1
      DO 7 I=IPIVC,I3
    7 A(I)=-A(I)*PIVOT
      A(IPIVC1)=PIVOT
C        PIVOT ELEMENT TRANSFORMED
C
C        NOW CONVERT REST OF THE MATRIX
      I1=MAIN-IDIM
C        POINTER TO PIVOT LINE ELEMENTS
      ICOL=1-IDIM
C        GENERAL COLUMN POINTER
      DO 10 I=1,IEMAT
      ICOL=ICOL+IDIM
      I1=I1+IDIM
C        POINTERS MOVED
      IF(I-MAIN) 8,10,8
C        PIVOT COLUMN EXCLUDED
    8 JCOL=ICOL+NMIN1
      SWAP=A(I1)
      I3=IPIVC-1
      DO 9 I2=ICOL,JCOL
      I3=I3+1
    9 A(I2)=A(I2)+SWAP*A(I3)
      A(I1)=SWAP*PIVOT
   10 CONTINUE
   11 CONTINUE
C        NOW REARRANGE THE MATRIX TO GET RIGHT INVERS
      DO 14 I1=1,N
      MAIN=N+1-I1
      LPIV=INDEX(MAIN)
      IF(LPIV-MAIN) 12,14,12
   12 ICOL=(LPIV-1)*IDIM+1
      JCOL=ICOL+NMIN1
      IPIVC=(MAIN-1)*IDIM+1-ICOL
      DO 13 I2=ICOL,JCOL
      I3=I2+IPIVC
      SWAP=A(I2)
      A(I2)=A(I3)
   13 A(I3)=SWAP
   14 CONTINUE
      DETERM=DETER
      NERROR=0
      RETURN
   15 NERROR=MAIN
      DETERM=DETER
      RETURN
      END

c*
C***************************************************************
C***************************************************************
c	